@@ -6,7 +6,7 @@
 ##
 
 export IN=.
-export OUT=$HOME/xy/hashcat-7.1.1
+export OUT=$HOME/xy/hashcat-7.1.2
 
 rm -rf $OUT
 rm -rf $OUT.7z
@@ -15,6 +15,8 @@ mkdir -p $OUT
 
 mkdir -p $OUT/tools
 mkdir -p $OUT/Python
+mkdir -p $OUT/Rust/generic_hash/target/release
+mkdir -p $OUT/Rust/generic_hash/target/x86_64-pc-windows-gnu/release
 
 cp    $IN/hashcat.exe                   $OUT/
 cp    $IN/hashcat.bin                   $OUT/
@@ -33,6 +35,11 @@ cp    $IN/example.dict                  $OUT/
 cp    $IN/example[0123456789]*.hash     $OUT/
 cp    $IN/example[0123456789]*.cmd      $OUT/
 cp -r $IN/Python/*.py                   $OUT/Python/
+cp -r $IN/Rust/generic_hash/src         $OUT/Rust/generic_hash/
+cp    $IN/Rust/generic_hash/Cargo.*     $OUT/Rust/generic_hash/
+cp    $IN/Rust/generic_hash/build.rs    $OUT/Rust/generic_hash/
+cp    $IN/Rust/generic_hash/target/release/*.so $OUT/Rust/generic_hash/target/release/
+cp    $IN/Rust/generic_hash/target/x86_64-pc-windows-gnu/release/*.dll $OUT/Rust/generic_hash/target/x86_64-pc-windows-gnu/release/
 cp -r $IN/OpenCL                        $OUT/
 cp    $IN/tools/*hashcat.pl             $OUT/tools/
 cp    $IN/tools/*hashcat.py             $OUT/tools/
@@ -94,6 +101,8 @@ chmod 755 $OUT/extra/tab_completion/*.sh
 chmod 755 $OUT/extra/tab_completion/install
 chmod 755 $OUT/Python
 chmod 644 $OUT/Python/*
+chmod 755 $OUT/Rust
+#chmod 644 $OUT/Python/*
 chmod 755 $OUT/OpenCL
 chmod 644 $OUT/OpenCL/*
 chmod 755 $OUT/tunings
